% Copyright 2013 Nicolas Pannetier, Cl�ment Debacker, Franck Mauconduit, Thomas Christen, Emmanuel Barbier
% This file is part of DCESIM.
% 
%   DCESIM is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
% 
%    DCESIM is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with DCESIM.  If not, see <http://www.gnu.org/licenses/>.

function [S, C, t] = VoxelSim(File)

% This code simulates the MR signal of DCE-like experiments
%
% Input:
%   - File  : Path to the parameter file
% Output:
%   - S     : MR Signal
%   - C     : CA concentration
%   - t     : Acquisition time
%
% Code by N.Pannetier, C.Debacker, INSERM, Grenoble, 2012

%%  INIT Simulation
%%###################
%% Load Input Parameters
[Model, Seq] = ReadModelAndSeq(File);

%% Define random number seed
stream = RandStream('mt19937ar');
reset(stream,Model.geo.vasc.Id*1e6+Model.geo.cell.Id);

%% NMR SEQUENCE
Pulseq = eval(Seq.Name);

%% Other paramters
Model.phy.gamma = 2.675*10^8;
Model.phy.vasc.khi = 4*pi*Model.phy.vasc.khi0*Model.phy.vasc.Hct*(1-Model.phy.vasc.Y);
tic_dt = 0;
tic

%%  START Simulation
%%###################

%% Generate Geometry lattice
G = GeoGen(Model);

%% Generate static matrices
% For the voxel
[too, CA, M] = InitSim(Model,G);
% For the MR sequence
too = InitSeq(Seq,too);

%% Other Paramters
count = 1;
C = [];
S = [];
t = [];
str = [];
AlreadyDraw = 0;

%% Main Time Loop
for tt=0:Model.dt:Model.Tmax
    
    %% Print progression
    if Model.Flag.verbose
        [str, tic_dt] = DisplayProgress(tt,Model,str,tic_dt);
    end
    
    %% Display real time lattice M, [CA] and S
    if Model.Flag.display
        if ~AlreadyDraw
            FigProp = DisplayFig_gen(G,M,CA,S,C,t);
            AlreadyDraw = 1;
        else
            FigProp = DisplayFig_up(G,M,CA,S,C,t,FigProp,tt);
        end
    end
    
    %% Physiology block
    CA = PhySim(Model,tt,too,G,CA);
    
    %% NMR block
    [M, Seqflag] = NMRSim(Model,too,G,CA,M,Pulseq,Seq,tt);
    
    %% Acquisition
    if Seqflag.Acq
        t(count) = tt;
        S(count) = sum(M.per(:));
        C(count) = mean(CA(:));
        count = count+1;
    end
    
    
end
